//Variable length GA
package multipath;

import java.util.*;
import java.text.*;
import java.io.*;

public class AppGA extends Multipath implements GeneticAlgorithm {

  public static int generationNumber;
  public static double bestScoreToDate;
  public static double bestScoreThisGen;
  public static int [] bestSolutionToDate;
  public static int [] bestSolutionThisGen;

  private int [] child1;
  private int [] child2;

  public AppGA() {
  }

  public void run(){
    while((generationNumber<numberGenerations)){
      step();
    }
  }

  public void step(){
    if(generationNumber<numberGenerations){
      generateNextGeneration();
      generationNumber++;
      updateBestSolution();
      output();
    }
  }

  public void init(){
    child1=new int[stringSize+1];
    child2=new int[stringSize+1];
    bestSolutionToDate=new int[stringSize+1];
    bestSolutionThisGen=new int[stringSize+1];
    generationNumber=0;
    bestScoreToDate=0;
    bestScoreThisGen=0;
    generateInitialGeneration();
    updateBestSolution();
  }

  //generation vector store each generation
  //each generation stored in array [][]
  //each chromosome stored in array []
  public void generateInitialGeneration(){
    int [] [] g0=new int[populationSize+1][stringSize+1];
    for(int n=1;n<=populationSize;n++){
      int [] p=new int[stringSize+1];
      for(int i=1;i<=stringSize;i++){
        p[i]=0;
      }
      //variable length of chromosone
      int length=1+rnd.nextInt(stringSize);
      for(int index=1;index<=length;index++){
        int temp=1+rnd.nextInt(maxServer);
        while(find(p,temp)!=-1){
          temp=1+rnd.nextInt(maxServer);
        }
        p[index]=temp;
      }
      g0[n]=p;
    }
    generation.addElement(g0);
  }

//generate next generation
  public void generateNextGeneration(){
    int [] [] previous =(int [] [])generation.lastElement();
    int [] [] next = new int[populationSize+1][stringSize+1];
    int [] [] matingPool =new int[populationSize+1][stringSize+1];
    //tournament
    for(int numMatingPool=1;numMatingPool<=populationSize;numMatingPool++){
      Vector picked =new Vector();
      BitSet included=new BitSet(populationSize+1);
      int numIncluded=0;

      do{
        int choice=1+rnd.nextInt(populationSize);
        while(included.get(choice)==true)
          choice=1+rnd.nextInt(populationSize);
        included.set(choice);
        numIncluded++;
        picked.addElement(previous[choice]);
      }while(numIncluded<tournament);

      //find the best among tournament and add to pool
      int bestIndex=0;
      double best=fitness((int [])picked.elementAt(0));
      for(int index=1;index<tournament;index++){
        if(fitness((int [])picked.elementAt(index))>best){
          bestIndex=index;
          best=fitness((int [])picked.elementAt(index));
        }
      }
      matingPool[numMatingPool]=(int [])picked.elementAt(bestIndex);
    }//End For numMatingPool

    BitSet included=new BitSet(populationSize+1);
    for(int n=1;n<populationSize;n=n+2){
      //mate (Crossover)
      if(rnd.nextDouble()<=matingProb){
        int choice1=1+rnd.nextInt(populationSize);
        while(included.get(choice1)==true)
          choice1=1+rnd.nextInt(populationSize);
        included.set(choice1);
        int choice2=1+rnd.nextInt(populationSize);
        while(included.get(choice2)==true)
          choice2=1+rnd.nextInt(populationSize);
        included.set(choice2);
        reproduce(matingPool[choice1],matingPool[choice2]);
        next[n]=child1;
        next[n+1]=child2;
      }
      //clone
      else{
        next[n]=matingPool[n];
        next[n+1]=matingPool[n+1];
      }
      //mutation
      if(rnd.nextDouble()<=mutationProb)
          mutation(next[n]);
      if(rnd.nextDouble()<=mutationProb)
          mutation(next[n+1]);
    }//end for n
  //copy the best solution into next generation
    next[populationSize]=bestSolutionToDate;
    generation.addElement(next);
  }

  private void reproduce(int [] p1, int [] p2){
    child1= new int[stringSize+1];
    child2= new int[stringSize+1];
    crossover(p1,p2);
  }

//fitness function
  private double fitness(int [] p){
    path(p);
    return getSpeed(p);
  }

  //calc path avail speed for each server in select[caseNum+1]
  public void path(int [] s){
    int [] select=new int[stringSize+1];
    select=s;
    //init
    for(int i=1;i<=maxServer;i++){
      actSpeed[i]=0;
    }
    for(int i=1;i<=maxPath;i++){
      pathes[i].availSpeed=pathes[i].speed;
    }
    //start update path avail speed for each server in select
    for(int sid=1;sid<=getLength(select);sid++){
        updatePath(select[sid]);
    }
  }

  //update available speed of path for server with server number s
  private void updatePath(int s){
    int sid=s;
    boolean pathEnd=false;
    double speed=servers[sid].speed;
    Node fNode=servers[sid].fatherNode;
    Path fPath=findPath(servers[sid],fNode);
    if((fPath.availSpeed - speed)>=0)
      fPath.availSpeed -= speed;
    else
      speed=fPath.availSpeed;

    Node cNode;
    while(!pathEnd){
      cNode=fNode;
      fNode=cNode.fatherNode;
      fPath=findPath(cNode,fNode);
      if((fPath.availSpeed - speed)>0)
        fPath.availSpeed -= speed;
      else{
        speed=fPath.availSpeed;
        fPath.availSpeed=0;
      }

      actSpeed[sid]=speed;
      if(fNode.isEqual(nodes[maxNode]))
        pathEnd=true;
    }
  }

//get total available speed for array s[]
  public double getSpeed(int [] s){
    int maxSpeed=0;
    int [] select=new int[stringSize+1];
    select=s;
    for(int i=1;i<=getLength(s);i++){
      maxSpeed += actSpeed[select[i]];
    }
    return maxSpeed;
  }

  public void updateBestSolution(){
    int [][] latest=(int [] [])generation.lastElement();
    bestScoreThisGen=fitness(latest[1]);
    bestSolutionThisGen=latest[1];
    for(int index=1;index<=populationSize;index++){
      if(fitness(latest[index])>bestScoreThisGen){
        bestScoreThisGen=fitness(latest[index]);
        for(int i=1;i<=stringSize;i++)
          bestSolutionThisGen[i]=latest[index][i];
      }
      else if((fitness(latest[index])==bestScoreThisGen)&&(getLength(latest[index])<getLength(bestSolutionThisGen))){
        bestScoreThisGen=fitness(latest[index]);
        for(int i=1;i<=stringSize;i++)
          bestSolutionThisGen[i]=latest[index][i];
      }
    }

    if(bestScoreToDate<bestScoreThisGen){
      bestScoreToDate=bestScoreThisGen;
      for(int i=1;i<=stringSize;i++)
        bestSolutionToDate[i]=bestSolutionThisGen[i];
    }
    else if((bestScoreToDate==bestScoreThisGen)&&(getLength(bestSolutionToDate)>getLength(bestSolutionThisGen))){
      bestScoreToDate=bestScoreThisGen;
      for(int i=1;i<=stringSize;i++)
        bestSolutionToDate[i]=bestSolutionThisGen[i];
    }

  }

  public void output(){
    System.out.print("\nGeneration "+generationNumber+". ");
    System.out.print("Best Score this Gen: "+bestScoreThisGen+". ");
    System.out.print("Best Solution: "+outputResult(bestSolutionThisGen)+". ");
    System.out.print("Best Score: "+bestScoreToDate+". ");
    System.out.print("Best Solution: "+outputResult(bestSolutionToDate)+". ");
  }

//output solution array
  private String outputResult(int [] s){
    String temp="";
    int [] sorted=new int [stringSize+1];
    sorted=s;
    BubbleSort(sorted);
    for(int i=1;i<sorted.length;i++){
      if(sorted[i]!=0)
        temp=temp+(new Integer(sorted[i])).toString()+",";
    }
    return temp;
  }

  private static void BubbleSort(int[] a)
  {
      int i,j;
      for (i=getLength(a)+1; --i >=0;) {
         int swapped = 0;
         for (j=1; j<i;j++) {
            if (a[j]>a[j+1]) {
              int T = a[j];
              a[j] = a[j+1];
  	      a[j+1] = T;
              swapped=1;
            }
         }
         if (swapped==0) return;
     }
  }

//variable length crossover method, make sure no duplicate, no exceeding max length
  private void crossover(int [] p1, int [] p2){
    int index1=1+rnd.nextInt(getLength(p1));
    int index2=1+rnd.nextInt(getLength(p2));
    for(int i=1;i<=stringSize;i++){
      child1[i]=0;
      child2[i]=0;
    }
    for(int i=1;i<index1;i++){
      child1[i]=p1[i];
    }
    for(int i=1;i<index2;i++){
      child2[i]=p2[i];
    }
    for(int i=index1;i<=(index1+getLength(p2)-index2+1);i++){
      if((i<=stringSize)&&(i-index1+index2<=stringSize))
        child1[i]=p2[i-index1+index2];
    }
    for(int i=index2;i<=(index2+getLength(p1)-index1+1);i++){
      if((i<=stringSize)&&(i-index2+index1<=stringSize))
        child2[i]=p1[i-index2+index1];
    }

    removeDuplicate(child1);
    removeDuplicate(child2);
  }

//remove duplicate
  private void removeDuplicate(int [] p){
    int index=checkDuplicate(p);
    int [] temp=new int [stringSize+1];
    temp=p;
    while(index!=-1){
      for(int i=index;i<=getLength(p);i++){
        if(i<stringSize)
          p[i]=temp[i+1];
        else
          p[i]=0;
      }
      index=checkDuplicate(p);
      temp=p;
    }
  }

//check duplicate
  private int checkDuplicate(int [] p){
    for(int i=1;i<=getLength(p);i++){
      for(int j=i+1;j<=getLength(p);j++){
        if((p[i]==p[j])&&(p[i]!=0))
          return i;
      }
    }
    return -1;
  }

//mutation:replace one bit with other number, make sure no duplicate
  private void mutation(int [] p){
    int index=1+rnd.nextInt(getLength(p));
    int value=1+rnd.nextInt(maxServer);
    p[index]=value;
    removeDuplicate(p);
  }

//find if value in array child[]
  private int find(int [] child, int value){
    for(int i=1;i<=stringSize;i++)
      if(child[i]==value)
        return i;
    return -1;
  }

//return no zero length of an array
  private static int getLength(int [] a){
    int length=0;
    for(int i=1;i<a.length;i++){
      if(a[i]!=0)
        length++;
    }
    return length;
  }

  //find a path with start node and end node
  private Path findPath(Node sNode, Node fNode){
    for(int i=1;i<=maxPath;i++){
      if((pathes[i].startNode.isEqual(sNode))&&(pathes[i].endNode.isEqual(fNode)))
        return pathes[i];
    }
    return null;
  }

  //format double number for output
  private static String formatNumber(double number) {
      DecimalFormat myFormatter = new DecimalFormat("#0.00#");
      return myFormatter.format(number);
  }

  public static void main(String[] args) {
    AppGA ga;
    LoadData.loadData();
    ga = new AppGA();
    ga.init();
    ga.run();
  }
}